#include "ai_ids_ch4_n2.hpp"

gsl_sf_legendre_t AI_IDS_ch4_n2::norm = GSL_SF_LEGENDRE_NONE;

std::vector<double> AI_IDS_ch4_n2::LegP1;
std::vector<double> AI_IDS_ch4_n2::LegP2;

std::complex<double> AI_IDS_ch4_n2::JJ  = std::complex<double>(0.0, 1.0);

void AI_IDS_ch4_n2::init() {
    size_t size1 = gsl_sf_legendre_array_n(lmax_bond);
    size_t size2 = gsl_sf_legendre_array_n(lmax_n2);

    LegP1.resize(size1);
    LegP2.resize(size2);
}

void AI_IDS_ch4_n2::fill_legP1(const double theta1) {
    double cos_theta1 = std::cos(theta1);
    gsl_sf_legendre_array_e(norm, lmax_bond, cos_theta1, csphase, &LegP1[0]);
}

void AI_IDS_ch4_n2::fill_legP2(const double theta2) {
    double cos_theta2 = std::cos(theta2);
    gsl_sf_legendre_array_e(norm, lmax_n2, cos_theta2, csphase, &LegP2[0]);
}

void AI_IDS_ch4_n2::ids(Eigen::Ref<Eigen::Vector3d> result, double R, double phi1, double theta1, double phi2, double theta2) {
    static Eigen::Vector3cd cmplx;
       
    fill_legP1(theta1);
    fill_legP2(theta2);

    if (R < 4.5) {
        std::cerr << "CALLING R = " << R << " WHICH IS NOT SUPPORTED.\n";
        std::exit(1);
    } else if (R < 4.75) {
        ids_p1(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 5.0) {
        ids_p2(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 5.25) {
        ids_p3(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 5.5) {
        ids_p4(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 5.75) {
        ids_p5(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 6.0) {
        ids_p6(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 6.25) {
        ids_p7(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 6.5) {
        ids_p8(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 6.75) {
        ids_p9(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 7.0) {
        ids_p10(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 7.25) {
        ids_p11(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 7.5) {
        ids_p12(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 7.75) {
        ids_p13(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 8.0) {
        ids_p14(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 8.25) {
        ids_p15(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 8.5) {
        ids_p16(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 8.75) {
        ids_p17(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 9.0) {
        ids_p18(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 9.25) {
        ids_p19(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 9.5) {
        ids_p20(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 10.0) {
        ids_p21(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 10.5) {
        ids_p22(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 11.0) {
        ids_p23(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 12.0) {
        ids_p24(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 14.0) {
        ids_p25(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 16.0) {
        ids_p26(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 18.0) {
        ids_p27(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 20.0) {
        ids_p28(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 25.0) {
        ids_p29(cmplx, R, phi1, theta1, phi2, theta2);
    } else if (R < 30.0) {
        ids_p30(cmplx, R, phi1, theta1, phi2, theta2);
    } else {
        ids_p31(cmplx, R, phi1, theta1, phi2, theta2);
    }

    result = cmplx.real();
} 
