#include <iostream>
#include <iomanip>

#include "ai_pes_ch4_n2.hpp"
#include "ai_ids_ch4_n2.hpp"

int main()
{
    std::cout << std::fixed << std::setprecision(16);
    
    const double DEG = M_PI / 180.0;

    double R      = 6.775;
    double PHI1   = 47.912 * DEG;
    double THETA1 = 56.167 * DEG;
    double PHI2   = 0.0    * DEG;
    double THETA2 = 135.0  * DEG;

    std::cout << " SELECTED GEOMETRY OF THE COMPLEX:\n";
    std::cout << " [SEE MAIN TEXT OF THE PAPER FOR THE DEFINITION OF THE BODY-FIXED FRAME]\n";
    std::cout << " INTERMOLECULAR DISTANCE R = " << R << " a0\n";
    std::cout << " ORIENTATION OF THE INTEMOLECULAR AXIS: (" << PHI1 << ", " << THETA1 << ")\n";
    std::cout << " ORIENTATION OF THE N2 MOLECULE: (" << PHI2 << ", " << THETA2 << ")\n";
    std::cout << " SELECTED GEOMETRY CORRESPONDS TO ONE OF THE EQUIVALENT GLOBAL MINIMA OF THE POTENTIAL ENERGY SURFACE\n\n\n";

    AI_PES_ch4_n2 pes;
    pes.init();

    pes.fill_legP1(THETA1);
    pes.fill_legP2(THETA2);

    std::cout << " POTENTIAL ENERGY SURFACE:\n";
    std::cout << " =====================================================\n";
    std::cout << " [U]: \t\t" << pes.pes(R, PHI1, THETA1, PHI2, THETA2) << " cm-1 \n\n\n";

    AI_IDS_ch4_n2 ids;
    ids.init();
   
    Eigen::Vector3d res;
    ids.ids(res, R, PHI1, THETA1, PHI2, THETA2);
    
    std::cout << "INDUCED DIPOLE SURFACE:\n";
    std::cout << " =====================================================\n";
    std::cout << " [DIPX]: \t" << res(0) << " (ea_0) \n";
    std::cout << " [DIPY]: \t" << res(1) << " (ea_0) \n";
    std::cout << " [DIPZ]: \t" << res(2) << " (ea_0) \n";

    return 0;
}
